@@ -25,81 +25,58 @@ from typing import Dict, List
 
 
 
-
-
-# # REMOVE THIS
-# def embed_subsystem(language, system_prototype, ouput_signals_name=None, calculate_outputs = True, update_states = False ):
-#     """  
-#         generate code to call a subsystem (TODO: this is obsolete remove it!)
-
-#         - system_prototype   - the block prototype including the subsystem - type: : dy.GenericSubsystem
-#         - ouput_signals_name - list of variable names to which the output signals of the subsystem are assigned to
-#         - calculate_outputs  - generate a call to the output computation API function of the subsystem
-#         - update_states      - generate a call to the state update API function of the subsystem
-#     """
-
-
-#     lines = '{ // subsystem ' + system_prototype.embedded_subsystem.name + '\n'
-
-#     innerLines = ''
-
-#     #
-#     # system_prototype is of type GenericSubsystem. call the code generation routine of the subsystem
-#     #
-
-#     # generate code for calculating the outputs 
-#     if calculate_outputs:
-
-#         innerLines += system_prototype.generate_code_output_list(language, system_prototype.outputs)
-
-#         if len(system_prototype.outputs) != len(ouput_signals_name):
-#             raise BaseException('len(system_prototype.outputs) != len(ouput_signals_name)')
-
-
-
-
-#         for i in range( 0, len( ouput_signals_name ) ): # NOTE:
-#             innerLines += asign( system_prototype.outputs[i].name, ouput_signals_name[i] )
-
-#     # generate code for updating the states
-#     if update_states:
-#         innerLines += system_prototype.generate_code_update(language)
-
-
-#     lines += indent(innerLines)
-#     lines += '}\n'
-
-#     return lines
-
-
-
-
-
-
-
-
-
-class TruggeredSubsystem(SingleSubsystemEmbedder):
+class TriggeredSubsystem(SingleSubsystemEmbedder):
     """
         Include a triggered sub-system
 
-        Optional:
+        Optional argument:
 
             prevent_output_computation = True: 
-                The subsystems outputs are only computed when triggered. Please note that the outputs
-                of the subsystem are uninitialized until the subsystem is triggered for the first time.
+                The subsystems outputs are only computed when triggered. Please note that, in this case,
+                the outputs of the subsystem is uninitialized until the subsystem is triggered for 
+                the first time.
 
     """
 
-    def __init__(self, sim : System, control_input : Signal, subsystem_prototype : GenericSubsystem,  prevent_output_computation = False ):
+    def __init__(self, system : System, control_input : Signal, subsystem_wrapper : SystemWrapper,  prevent_output_computation = False ):
         
         self._control_input = control_input
         self.prevent_output_computation = prevent_output_computation
 
-        SingleSubsystemEmbedder.__init__(self, sim, 
-                                        control_inputs=[control_input], 
-                                        subsystem_prototype=subsystem_prototype, 
-                                        number_of_control_outputs=0 )
+        SingleSubsystemEmbedder.__init__(
+            self, system, 
+            control_inputs=[control_input], 
+            subsystem_wrapper=subsystem_wrapper, 
+            number_of_control_outputs=0
+        )
+
+    # TODO: rename to direct feedthrough
+    def config_request_define_feedforward_input_dependencies(self, outputSignal):
+
+        # determine inputs needed directly to compute the outputs
+        required_inputs = set()
+        required_inputs.update(self._subsystem_wrapper.inputs_to_calculate_outputs)
+
+        # the trigger input signal self._control_input is only used for state updates
+        # and, hence, not required for computing the outputs of the if-subsystem block.
+        #
+        # If, however, the flag self.prevent_output_computation is activated,
+        # the outputs are directly influenced by the trigger signals. Therefore,
+        # it is added to the list of required inputs.
+
+        if self.prevent_output_computation:
+            required_inputs.update([self._control_input])
+
+        return list(required_inputs)
+
+    def config_request_define_state_update_input_dependencies(self, outputSignal):
+
+        # determine inputs needed to update the states of the subsystem
+        required_inputs = set()
+        required_inputs.update(self._subsystem_wrapper.inputs_to_update_states)
+        required_inputs.update([self._control_input])
+
+        return list(required_inputs)
 
 
     def generate_code_output_list(self, language, signals : List [ Signal ] ):
@@ -108,12 +85,13 @@ class TruggeredSubsystem(SingleSubsystemEmbedder):
         if language == 'c++':
             
             # generate code to call subsystem output(s)
-            code_compute_output = embed_subsystem3(
+            code_compute_output = embed_subsystem(
                 language, 
-                subsystem_prototype=self._subsystem_prototype, 
-                assign_to_signals=signals, 
-                ouput_signals_of_subsystem=self.outputs_map_from_embedding_block_to_subsystem.map( signals ),
-                calculate_outputs = True, update_states = False 
+                system_wrapper                         = self._subsystem_wrapper, 
+                assign_to_signals                      = signals, 
+                indices_of_output_signals_of_subsystem = self.outputs_map_from_embedding_block_to_subsystem.map_to_output_index( signals ),
+                calculate_outputs                      = True,
+                update_states                          = False 
             )
 
             if self.prevent_output_computation:
@@ -138,10 +116,11 @@ class TruggeredSubsystem(SingleSubsystemEmbedder):
         if language == 'c++':
 
             # generate code to compute the state update of the subsystem
-            code_compute_state_update = embed_subsystem3(
+            code_compute_state_update = embed_subsystem(
                 language, 
-                subsystem_prototype=self._subsystem_prototype, 
-                calculate_outputs = False, update_states = True
+                system_wrapper    = self._subsystem_wrapper, 
+                calculate_outputs = False,
+                update_states     = True
             )
 
             # the subsystems update is only performed when triggered
@@ -158,30 +137,34 @@ class TruggeredSubsystem(SingleSubsystemEmbedder):
 
 class LoopUntilSubsystem(SingleSubsystemEmbedder):
     """
-        Embed a loop sub-system
-
+        Embed a sub-system wrapped into a loop
     """
 
-    def __init__(self, sim : System, max_iterations : int, 
-                    subsystem_prototype : GenericSubsystem, 
-                    until_signal : Signal = None, yield_signal : Signal = None ):
+    def __init__(self, system : System, max_iterations : int, 
+                    subsystem_wrapper : SystemWrapper, 
+                    add_until_control : Signal = False, add_yield_control : Signal = False ):
         
         self.max_iter = max_iterations
-        self._until_signal = until_signal
-        self._yield_signal = yield_signal
+        self._add_until_control = add_until_control
+        self._add_yield_control = add_yield_control
 
         number_of_control_outputs = 0
-        if self._until_signal is not None:
+        if self._add_until_control:
             number_of_control_outputs += 1
 
-        if self._yield_signal is not None:
+        if self._add_yield_control:
             number_of_control_outputs += 1
 
+        # could also add extra output signals 'until_output' and 'yield_output' and mark them with 
+        # system.add_signal_mandatory_to_compute( self.until_output )
+        # system.add_signal_mandatory_to_compute( self.yield_output )
 
-        SingleSubsystemEmbedder.__init__(self, sim, 
-                                        control_inputs=[],   # TODO: add N_iter signal here 
-                                        subsystem_prototype=subsystem_prototype, 
-                                        number_of_control_outputs=number_of_control_outputs )
+        SingleSubsystemEmbedder.__init__(
+            self, system, 
+            control_inputs=[],   # TODO: add N_iter signal here 
+            subsystem_wrapper=subsystem_wrapper, 
+            number_of_control_outputs=number_of_control_outputs
+        )
 
 
     def generate_code_output_list(self, language, signals : List [ Signal ] ):
@@ -189,72 +172,84 @@ class LoopUntilSubsystem(SingleSubsystemEmbedder):
         lines = ''
         if language == 'c++':
 
-            assign_to_signals = []
-            ouput_signals_of_subsystem = []
+            assign_to_variables = []
+            indices_of_output_signals_of_subsystem = []
 
-            assign_to_signals.extend(signals)
-            ouput_signals_of_subsystem.extend(self.outputs_map_from_embedding_block_to_subsystem.map( signals )) # output signal mapping lookup
+            assign_to_variables.extend( cgh.signal_list_to_name_list(signals) )
 
-            control_output_index = 0
+            indices_of_output_signals_of_subsystem.extend(
+                self.outputs_map_from_embedding_block_to_subsystem.map_to_output_index( signals )
+            ) # output signal mapping lookup
+
+
+            # The outputs of the embedded system are referred to by indices.
+            # In this case, count the control outputs that start after the normal outputs
+            control_output_index = self.number_of_normal_outputs
+
+            if self._add_until_control:
 
-            if self._until_signal is not None:
                 # define tmp-var for self._until_signal instead of a block output
-                lines += cgh.defineVariables( [self._until_signal] )
+                lines += 'bool _until_condition;\n'
 
                 # add list of signals to assign
-                assign_to_signals.append( self._until_signal )
-                ouput_signals_of_subsystem.append( self._control_signals_from_embeded_system[control_output_index] )
+                assign_to_variables.append( '_until_condition' )
+                indices_of_output_signals_of_subsystem.append( control_output_index )
             
                 control_output_index += 1
 
 
-            if self._yield_signal is not None:
-                # define tmp-var for self._yield_signal instead of a block output
-                lines += cgh.defineVariables( [self._yield_signal] )
+            if self._add_yield_control:
 
+                # define tmp-var for self._yield_signal instead of a block output
+                lines += 'bool _yield_condition;\n'
                 # add list of signals to assign
-                assign_to_signals.append( self._yield_signal )
-                ouput_signals_of_subsystem.append( self._control_signals_from_embeded_system[control_output_index] )
+                assign_to_variables.append( '_yield_condition' )
+                indices_of_output_signals_of_subsystem.append( control_output_index )
 
                 control_output_index += 1
 
 
 
-            calc_outputs = embed_subsystem3(
+            calc_outputs = embed_subsystem(
                 language, 
-                subsystem_prototype=self._subsystem_prototype, 
-                assign_to_signals=assign_to_signals, 
-                ouput_signals_of_subsystem=ouput_signals_of_subsystem,
+                system_wrapper=self._subsystem_wrapper, 
+                assign_to_variable_names=assign_to_variables, 
+                indices_of_output_signals_of_subsystem=indices_of_output_signals_of_subsystem,
                 calculate_outputs = True, update_states = False
             )
 
             
 
-            update_states = embed_subsystem3(
+            update_states = embed_subsystem(
                 language, 
-                subsystem_prototype=self._subsystem_prototype, 
+                system_wrapper=self._subsystem_wrapper, 
                 calculate_outputs = False, update_states = True
             )
 
+            code_reset_subsystem = embed_subsystem(
+                language, 
+                system_wrapper=self._subsystem_wrapper, 
+                calculate_outputs = False, update_states = False,
+                reset_states = True
+            )
+
             code = ''
             code +=  calc_outputs 
             code += update_states
 
-            if self._yield_signal is not None:
-                code += cgh.generate_loop_break(language, condition=self._yield_signal.name)
+            if self._add_yield_control:
+                code += cgh.generate_loop_break(language, condition='_yield_condition')
 
-            if self._until_signal is not None:
+            if self._add_until_control:
 
-                code_reset_subsystem = embed_subsystem3(
-                    language, 
-                    subsystem_prototype=self._subsystem_prototype, 
-                    calculate_outputs = False, update_states = False,
-                    reset_states = True
-                )
+                code += cgh.generate_loop_break(language, condition='_until_condition', code_before_break=code_reset_subsystem)
 
-                code += cgh.generate_loop_break(language, condition=self._until_signal.name, code_before_break=code_reset_subsystem)
 
-            lines += cgh.generate_loop( language, max_iterations=str(self.max_iter), code_to_exec=code  )
+            # TODO: issue an error once the maximal number of iterations has passed.
+            lines += cgh.generate_loop( language, max_iterations=str(self.max_iter), code_to_exec=code, code_to_exec_on_abort = code_reset_subsystem  )
+
+            # wrap all into brackets to avoid naming conflicts in the defined variables
+            lines = cgh.brackets(lines) 
 
         return lines
 
@@ -262,11 +257,7 @@ class LoopUntilSubsystem(SingleSubsystemEmbedder):
 
         lines = ''
         if language == 'c++':
-
-            if self._until_signal is not None:
-                pass    
-                #lines += self._subsystem_prototype.generate_code_reset(language)
-
+            pass    
 
         return lines
 
@@ -281,14 +272,6 @@ class LoopUntilSubsystem(SingleSubsystemEmbedder):
 
 
 
-
-
-
-
-
-
-
-
 
 
 
@@ -299,15 +282,17 @@ class SwitchSubsystems(MultiSubsystemEmbedder):
         The outputs of the currently active subsystem are forwarded.
     """
 
-    def __init__(self, sim : System, control_input : Signal, subsystem_prototypes : List [GenericSubsystem], reference_outputs : List [Signal] ):
+    def __init__(self, system : System, control_input : Signal, subsystem_wrappers : List [SystemWrapper], reference_outputs : List [Signal] ):
         
         self._control_input = control_input
 
-        MultiSubsystemEmbedder.__init__(self, sim, 
-                                        control_inputs=[control_input], 
-                                        subsystem_prototypes=subsystem_prototypes, 
-                                        switch_reference_outputs=reference_outputs,
-                                        number_of_control_outputs=0 )
+        MultiSubsystemEmbedder.__init__(
+            self, system, 
+            control_inputs=[control_input], 
+            subsystem_wrappers=subsystem_wrappers, 
+            switch_reference_outputs=reference_outputs,
+            number_of_control_outputs=0
+        )
 
     def generate_code_defStates(self, language):
         lines = MultiSubsystemEmbedder.generate_code_defStates(self, language)
@@ -322,8 +307,7 @@ class SwitchSubsystems(MultiSubsystemEmbedder):
             lines += self.codegen_help_generate_switch( 
                 language=language,
                 switch_control_signal_name=self._control_input.name,
-                switch_ouput_signals=signals,
-                additional_outputs=[],
+                switch_output_signals=signals,
                 calculate_outputs = True, update_states = False
             )
 
@@ -358,24 +342,35 @@ class StatemachineSwitchSubsystems(MultiSubsystemEmbedder):
         self.state_output  - 
     """
 
-    def __init__(self, sim : System, subsystem_prototypes : List [GenericSubsystem], reference_outputs : List [Signal] ):
+    def __init__(
+            self, 
+            system                 : System, 
+            subsystem_wrappers     : List [SystemWrapper], 
+            reference_outputs      : List [Signal],
+            immediate_state_switch : bool                   = False
+        ):
         
-        MultiSubsystemEmbedder.__init__(self, sim, 
-                                        control_inputs=[], 
-                                        subsystem_prototypes=subsystem_prototypes, 
-                                        switch_reference_outputs=reference_outputs,
-                                        number_of_control_outputs = 1 )
+        MultiSubsystemEmbedder.__init__(
+            self, system, 
+            control_inputs            = [], 
+            subsystem_wrappers        = subsystem_wrappers, 
+            switch_reference_outputs  = reference_outputs,
+            number_of_control_outputs = 1
+        )
+
+        self._immediate_state_switch = immediate_state_switch
+        self._number_of_states       = len(subsystem_wrappers)
 
         # how to add more outputs?
-        self.state_output.setDatatype( dt.DataTypeInt32(1) )
+        self.state_control.setDatatype( dt.DataTypeInt32(1) )
 
         # this output signals must be compted in any way
         # also in case it is not used by other blocks
-        sim.add_signal_mandatory_to_compute( self.state_output )
+        system.add_signal_mandatory_to_compute( self.state_control )
 
 
     @property
-    def state_output(self):
+    def state_control(self):
         return self.control_outputs[0]
 
 
@@ -400,13 +395,93 @@ class StatemachineSwitchSubsystems(MultiSubsystemEmbedder):
 
     def generate_code_output_list(self, language, signals : List [ Signal ] ):
 
+        # NOTE: 'signals' automatically contains 'self.state_control' as it is marked with 'system.add_signal_mandatory_to_compute'
+
         lines = ''
         if language == 'c++':
 
-            lines += self.codegen_help_generate_switch( language=language, 
-                                            switch_control_signal_name=  self._state_memory ,
-                                            switch_ouput_signals=signals,
-                                            additional_outputs=[self.state_output] )
+            if not self._immediate_state_switch:
+
+                lines += self.codegen_help_generate_switch( 
+                    language                   = language, 
+                    switch_control_signal_name = self._state_memory,
+                    switch_output_signals      = signals
+                )
+
+            else:
+                
+                # The algorithm works as follows:
+                #
+                # int _state_to_try = 'self._state_memory'
+                # int _state_control_tmp;
+                # int _counter = 0;
+                #
+                # while(1) {
+                #
+                #   _state_control_tmp = 'calculate control output of subsystem identified by _state_to_try'
+                #
+                #   if ( 'self.state_control' == -1 and _counter == 0 ) break;
+                #   if ( 'self.state_control' == _state_to_try ) break;
+                #   if ( 'self.state_control' == -1 ) { 'self.state_control' = _state_to_try; break; }
+                #   
+                #   _state_to_try = 'self.state_control'
+                #
+                #   ++_counter;
+                #   if (_counter >= 'self._number_of_states') {
+                #     'raise error'
+                #     break;
+                #   }
+                #
+                # }
+                #
+                lines += 'int _state_to_try;\n'
+                lines += cgh.assign( self._state_memory, '_state_to_try' )
+
+                # loop
+                in_loop_lines = ''
+
+                in_loop_lines += self.codegen_help_generate_switch( 
+                    language                   = language, 
+                    switch_control_signal_name = '_state_to_try',
+                    switch_output_signals      = signals   # NOTE: signals includes self.state_control as it is marked as mandatory to be computed
+                )
+
+                # if ( 'self.state_control' == -1 and _counter == 0 ) break;
+                in_loop_lines += cgh.generate_loop_break(
+                    language, 
+                    condition = cgh.generate_and(
+                        language, 
+                        cgh.generate_compare_equality_to_constant( language, self.state_control.name, -1 ),
+                        cgh.generate_compare_equality_to_constant( language, '_counter', 0 )
+                    )
+                )
+
+                # if ( 'self.state_control' == _state_to_try ) break;
+                in_loop_lines += cgh.generate_loop_break(
+                    language, 
+                    condition = cgh.generate_compare_equality( language, self.state_control.name, '_state_to_try' )
+                )
+
+                # if ( 'self.state_control' == -1 ) { 'self.state_control' = _state_to_try; break; }
+                in_loop_lines += cgh.generate_loop_break(
+                    language, 
+                    condition         = cgh.generate_compare_equality_to_constant( language, self.state_control.name, -1 ),
+                    code_before_break = cgh.assign( '_state_to_try', self.state_control.name )
+                )
+
+                in_loop_lines += cgh.assign( self.state_control.name, '_state_to_try' )
+
+                lines += cgh.generate_loop(
+                    language,
+                    max_iterations        = self._number_of_states, 
+                    code_to_exec          = in_loop_lines,
+                    counter_variable_name = '_counter'
+                )
+                # end loop
+
+                # wrap all into brackets to avoid naming conflicts in the defined variables
+                lines = cgh.brackets(lines) 
+
 
         return lines
 
@@ -420,7 +495,7 @@ class StatemachineSwitchSubsystems(MultiSubsystemEmbedder):
         condition_list = []
 
         subsystem_counter = 0
-        for system_prototype in self._subsystem_prototypes:
+        for system_prototype in self._subsystem_wrappers:  # TODO: whats wrong here?
 
             code_reset_states = MultiSubsystemEmbedder.generate_reset( self, language, system_index=subsystem_counter ) 
 
@@ -431,7 +506,7 @@ class StatemachineSwitchSubsystems(MultiSubsystemEmbedder):
 
             subsystem_counter += 1
 
-        # combine conditions and their repective actions
+        # combine conditions and their respective actions
         lines += cgh.generate_if_else(language, condition_list, action_list)
 
         return lines
@@ -449,7 +524,7 @@ class StatemachineSwitchSubsystems(MultiSubsystemEmbedder):
             )
 
             # get the signal issued by the currently active subsystem that describes the requests for a stare transition
-            state_control_signal_from_subsystems = self.state_output
+            state_control_signal_from_subsystems = self.state_control
 
             # reset current subsystem in case a state transition is requested
             lines_reset_subsystem = self.generate_switch_to_reset_leaving_subsystem(language, self._state_memory )
