@@ -38,11 +38,11 @@ class CompileDiagram: # TODO: does this need to be a class? so far no.
     def __init__(self):
 
         # self._manifest = None
-        self._compleResults = None
+        self._comple_results = None
 
     @property
     def compileResults(self):
-        return self._compleResults
+        return self._comple_results
     
     def traverseSubSystems(self, system : System, level):
 
@@ -71,24 +71,31 @@ class CompileDiagram: # TODO: does this need to be a class? so far no.
 
 
 
-        compile_result = compile_single_system( system, reduce_uneeded_code = not is_top_level_system )
+        compile_result = compile_single_system(
+            system, 
+            reduce_not_needed_code = not is_top_level_system,
+            subsytem_nesting_level = level
+        )
 
 
-        # # produre commands for building/executing
+        # # procedure commands for building/executing
         # command_list_for_all_subsystems = []
         # for subsystem in system.subsystems:
         #     command_list_for_all_subsystems.append( subsystem.command_to_execute )
 
 
         # replace the execution command by one that wraps all subsystems along with the main system
-        execution_command = PutSystemAndSubsystems( command_to_put_main_system=compile_result.command_to_execute, commands_to_put_subsystems=command_list_for_all_subsystems )
+        execution_command = PutSystemAndSubsystems(
+            command_to_put_main_system = compile_result.command_to_execute, 
+            commands_to_put_subsystems = command_list_for_all_subsystems 
+        )
         compile_result.set_command_to_execute( execution_command )
 
         # store the compilation result in the system's structure (TODO: is this needed?)
         system.compile_result = compile_result
 
         if is_top_level_system:
-            self._compleResults = compile_result
+            self._comple_results = compile_result
 
         return compile_result
 
@@ -108,13 +115,13 @@ class CompileDiagram: # TODO: does this need to be a class? so far no.
         if main_compile_result is None:
             raise BaseException("failed to obtain the compilation results")
 
-        self._compleResults = main_compile_result
+        self._comple_results = main_compile_result
 
         return main_compile_result
 
 
 
-def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=0):
+def compile_single_system(system, reduce_not_needed_code = False, enable_print:int=0, subsytem_nesting_level : int = 0):
 
     # the primary output signals are the outputs of the compiled system
     outputSignals = system.primary_outputs
@@ -140,7 +147,7 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
 
     E=BuildExecutionPath()
 
-    print("determining the computation order...")
+    # print("determining the computation order...")
 
     # append all execution lines to this structure
     executionLineToCalculateOutputs = ExecutionLine( [], [], [], [], [] )
@@ -164,7 +171,7 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
         if blk.output_signals is not None:
             if len(blk.output_signals) == 0: # no output signals --> must be a sink
 
-                # print(Style.BRIGHT, "found a sink-type block", blk.name)
+                # print(Style.BRIGHT, "found a sink-type block in (sub)system", blk.name, system.name)
 
                 inputs_to_update_states_tmp = blk.getBlockPrototype().config_request_define_state_update_input_dependencies( None )
                 if inputs_to_update_states_tmp is not None:
@@ -206,13 +213,13 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
 
     for s in list(signals_to_compute):
 
-        elForOutputS = E.getExecutionLine( s )
+        elForOutputS = E.determine_execution_order( s, system )
 
         if enable_print > 1:
             elForOutputS.printExecutionLine()
 
         # merge all lines into one
-        # TODO use sets inside 'appendExecutionLine' some block are present twiche
+        # TODO use sets inside 'appendExecutionLine' some block are present twice
         executionLineToCalculateOutputs.appendExecutionLine( elForOutputS )
 
 
@@ -225,11 +232,11 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
 
 
 
-    print("building execution paths...")
+    # print("building execution paths...")
 
-    # look into executionLineToCalculateOutputs.dependencySignals and use E.getExecutionLine( ) for each
+    # look into executionLineToCalculateOutputs.dependencySignals and use E.determine_execution_order( ) for each
     # element. Also collect the newly appearing dependency signals in a list and also 
-    # call E.getExecutionLine( ) on them. Stop until no further dependend signal appear.
+    # call E.determine_execution_order( ) on them. Stop until no further dependend signal appear.
     # finally concatenare the execution lines
 
     # start with following signals to be computed
@@ -269,11 +276,14 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
 
     # build the API function calcPrimaryResults() that calculates the outputs of the simulation.
     # Further, it stores intermediate results
-    commandToPublishTheResults = PutAPIFunction("calcResults_1", 
-                                                inputSignals=simulationInputSignalsToCalculateOutputs,
-                                                outputSignals=outputSignals, 
-                                                executionCommands=[ commandToCalcTheResultsToPublish, commandToCacheIntermediateResults ],
-                                                generate_wrappper_functions = not reduce_uneeded_code )
+    commandToPublishTheResults = PutAPIFunction(
+        "calcResults_1", 
+        inputSignals=simulationInputSignalsToCalculateOutputs,
+        outputSignals=outputSignals, 
+        executionCommands=[ commandToCalcTheResultsToPublish, commandToCacheIntermediateResults ],
+        generate_wrappper_functions = not reduce_not_needed_code,
+        subsystem_nesting_level  = subsytem_nesting_level
+    )
 
     # Initialize the list of commands to execute to update the states
     commandsToExecuteForStateUpdate = []
@@ -317,7 +327,7 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
         for s in dependencySignalsThroughStates:
 
             # get execution line to calculate s
-            executionLineForS = E.getExecutionLine(s)
+            executionLineForS = E.determine_execution_order(s, system)
 
             # store this execution line
             executionLinesForCurrentOrder.append(executionLineForS)
@@ -384,11 +394,11 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
 
 
 
-        # create state update command and append to the list of commnds to execute for state-update
+        # create state update command and append to the list of commands to execute for state-update
         sUpCmd = CommandUpdateStates( blocksWhoseStatesToUpdate )
         commandsToExecuteForStateUpdate.append( sUpCmd )
 
-        # get the dependendy singals of the current order
+        # get the denpendent singals of the current order
         # TODO important: remove the signals that are already computable from this list
         #dependencySignals = executionLineForCurrentOrder.dependencySignals
         dependencySignalsSimulationInputs = executionLineForCurrentOrder.dependencySignalsSimulationInputs
@@ -396,19 +406,19 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
         dependencySignalsThroughStates    = executionLineForCurrentOrder.dependencySignalsThroughStates
 
 
-        # TODO: handle special case in which a simulation input is requried for the state update of a block
+        # TODO: handle special case in which a simulation input is required for the state update of a block
         #       and was before found to be required to calculate the outpus of sth. 
-        # instead of printing 'has already been calculated in a previous traversion' create an input to the update() function
+        # instead of printing 'has already been calculated in a previous traversal' create an input to the update() function
         #
         # --- signals needed *indirectly* for s30 (through state update) --
         # -> S osc_excitement
         # -> S s22
         # --- signals needed for s30 --
         # -> S osc_excitement
-        # .  has already been calculated in a previous traversion
+        # .  has already been calculated in a previous traversal
 
 
-        # find out which singnals must be further computed to allow a state-update of the blocks
+        # find out which signals must be further computed to allow a state-update of the blocks
         dependencySignals__ = dependencySignalsThroughStates + dependencySignalsSimulationInputs
 
         # add the system inputs needed to update the states
@@ -417,41 +427,50 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
         # iterate
         order = order + 1
         if len(dependencySignals__) == 0:
-            print(Fore.GREEN + "All dependencies are resolved.")
+            # print(Fore.GREEN + "All dependencies are resolved.")
 
             break
 
         if order == 1000:
-            raise BaseException(Fore.GREEN + "Maximal number of iterations reached -- this is likely because of an algebraic loop or your simulation is very complex")
+            raise BaseException(Fore.GREEN + "In system " +  system.name + ": the maximal number of iterations was reached. This is likely because of an algebraic loop.")
             break
 
 
 
+    # simulationInputSignalsToUpdateStates is a set. Now fix the order of the signals to be consisten
+    simulation_input_signals_to_update_states_fixed_list = list(simulationInputSignalsToUpdateStates)
 
     # Build API to update the states: e.g. c++ function updateStates()
-    commandToUpdateStates = PutAPIFunction( nameAPI = 'updateStates', 
-                                            inputSignals=list(simulationInputSignalsToUpdateStates), 
-                                            outputSignals=[], 
-                                            executionCommands=commandsToExecuteForStateUpdate,
-                                            generate_wrappper_functions = not reduce_uneeded_code )
+    commandToUpdateStates = PutAPIFunction(
+        nameAPI = 'updateStates', 
+        inputSignals                = simulation_input_signals_to_update_states_fixed_list, 
+        outputSignals               = [], 
+        executionCommands           = commandsToExecuteForStateUpdate,
+        generate_wrappper_functions = not reduce_not_needed_code,
+        subsystem_nesting_level     = subsytem_nesting_level
+    )
 
     # code to reset add blocks in the simulation
     commandsToExecuteForStateReset = CommandResetStates( blockList=blocksWhoseStatesToUpdate_All)
 
     # create an API-function resetStates()
-    commandToResetStates = PutAPIFunction( nameAPI = 'resetStates', 
-                                            inputSignals=[], 
-                                            outputSignals=[], 
-                                            executionCommands=[commandsToExecuteForStateReset],
-                                            generate_wrappper_functions = not reduce_uneeded_code )
+    commandToResetStates = PutAPIFunction(
+        nameAPI                     = 'resetStates', 
+        inputSignals                = [], 
+        outputSignals               = [], 
+        executionCommands           = [commandsToExecuteForStateReset],
+        generate_wrappper_functions = not reduce_not_needed_code,
+        subsystem_nesting_level     = subsytem_nesting_level
+    )
 
 
     # define the interfacing class
-    command_to_execute_system = PutSystem(    system = system,
-                                                    resetCommand = commandToResetStates, 
-                                                    updateCommand = commandToUpdateStates,
-                                                    outputCommand = commandToPublishTheResults
-                                                )
+    command_to_execute_system = PutSystem(
+        system = system,
+        resetCommand = commandToResetStates, 
+        updateCommand = commandToUpdateStates,
+        outputCommand = commandToPublishTheResults
+    )
 
     # collect all (needed) inputs to this system
     allinputs = set(( simulationInputSignalsToUpdateStates ))
@@ -464,7 +483,7 @@ def compile_single_system(system, reduce_uneeded_code = False, enable_print:int=
     compleResults = CompileResults( manifest, command_to_execute_system)
 
     compleResults.inputSignals                             = allinputs
-    compleResults.simulationInputSignalsToUpdateStates     = simulationInputSignalsToUpdateStates
+    compleResults.simulationInputSignalsToUpdateStates     = simulation_input_signals_to_update_states_fixed_list
     compleResults.simulationInputSignalsToCalculateOutputs = simulationInputSignalsToCalculateOutputs
     compleResults.outputSignals                            = outputSignals
 
