@@ -1,5 +1,7 @@
 from .signal_network.signals import *
 from .signal_network.Block import *
+# from .system import * 
+
 
 from typing import Dict, List
 from colorama import init,  Fore, Back, Style
@@ -142,7 +144,6 @@ class graph_traversion2:
 
 
 
-
 class ExecutionLine():
     """
         This is a data structure
@@ -260,123 +261,120 @@ class ExecutionLine():
 
 
 
+
 class BuildExecutionPath:
     """
         Find out the order in which signals have to be computed such that a given signal
-        'signalToCalculte'can be calculated. This means finding out all dependencies of
-        'signalToCalculte'. For each call to 'getExecutionLine' only the signals that
+        'signal_to_calculate' can be calculated. This means finding out all dependencies of
+        'signal_to_calculate'. For each call to 'determine_execution_order' only the signals that
         were not already marked as a dependency in previous calls are returned.
-        Each call to 'getExecutionLine' gives an instance 'ExecutionLine'
+        Each call to 'determine_execution_order' gives an instance 'ExecutionLine'
     """
     def __init__(self, show_print:int=0):
 
         self._show_print = show_print
 
-        # list of signals the computation depends on (the tips of the execution tree)
-        self.dependencySignals = []
-
-        self.dependencySignalsThroughStates = []
-
-        # the list of signals to compute in correct order 
-        self.execution_order = []
-
-        # the list of simulation input signals required for the computation
-        self.dependencySignalsSimulationInputs = []
-
-        # For each signgal self.execution_order there might be a blocks that
-        # has an internal memory. It is required to build a list of those blocks
-        # that need a state update after their output(s) are calculated.
-        self.blocksToUpdateStates = []
-
         # list of marked signals (important to reset their visited flags)
-        self.markedSignals = []
+        self.marked_signals = []
 
-        # number of calls to getExecutionLine()
+        # number of calls to determine_execution_order()
         self.level = 0
 
     def __del__(self):
         # reset the grap markers stored in the signals
-        self.resetMarkers()
+        self.reset_markers()
 
 
-    def getExecutionLine(self, signalToCalculte : Signal):
+    def determine_execution_order(self, signal_to_calculate : Signal, current_system):
 
         """
             get the order of computation steps and their order that have
-            to be performed to compute 'signalToCalculte'
+            to be performed to compute 'signal_to_calculate'
             
             For each call to this function, a list is generated that does not contain
             signals that are already part of a previous list (that are already computed)
             
-            This function can be called multiple times and returns only the necessaray 
+            This function can be called multiple times and returns only the necessary 
             computations. Computations already planned in previous calls of this function
-            are not listed again. (until resetMarkers() is called)
+            are not listed again. (until reset_markers() is called)
 
             -- results --
 
-            self.execution_order contains the list of signals to comute in the correct order including
+            execution_order contains the list of signals to comute in the correct order including
             the target signals. Not included in this list are signals that cross the border to the simulation
-            specified by signalToCalculte.system (coming from an outer system). Further, not included are
-            signals that have been computet in a previous call to getExecutionLine().
+            specified by signal_to_calculate.system (coming from an outer system). Further, not included are
+            signals that have been computet in a previous call to determine_execution_order().
 
-            self.dependencySignals contains all signals that are required to comput signalToCalculate
+            dependency_signals contains all signals that are required to comput signalToCalculate
             and either cross the border of a simulation, 
         """
 
-        # TODO: dependency signals should stay as theiy are but reachableSignals should not contain signals
+        # TODO: dependency signals should stay as they are but reachableSignals should not contain signals
         #       that already have been calculated. Further, reachableSignals shall also contain dependency if they 
         #       were not already calculated
 
         if self._show_print > 0:
-            print("getExecutionLine on level " + str(self.level) )
-
-        # reset the lists TODO: use sets instead to avoid duplication?
-        self.execution_order = []
-        self.dependencySignals = []
-        self.dependencySignalsThroughStates = []
-        self.dependencySignalsSimulationInputs = []
-        self.blocksToUpdateStates = []
-
-        # search within this system
-        self.system = signalToCalculte.sim
+            print("determine_execution_order on level " + str(self.level) )
 
         # compute by traversing the tree
-        self.backwardTraverseSignalsExec__(startSignal=signalToCalculte, depthCounter = 0)
+        execution_order, dependency_signals, dependency_signals_simulation_inputs, blocks_to_update_states, dependency_signals_through_states = self.backwards_traverse_signals_exec__(start_signal=signal_to_calculate, depth_counter = 0, current_system=current_system)
 
         # the iteration level
         self.level = self.level + 1
 
-        return ExecutionLine( self.execution_order, self.dependencySignals, self.dependencySignalsSimulationInputs, self.blocksToUpdateStates, self.dependencySignalsThroughStates )
+        return ExecutionLine( execution_order, dependency_signals, dependency_signals_simulation_inputs, blocks_to_update_states, dependency_signals_through_states )
 
     def printExecutionLine(self):
         pass
 
-    def resetMarkers(self):
+    def reset_markers(self):
         # reset graph traversion markers
-        for signal in self.markedSignals:
+        for signal in self.marked_signals:
             signal.graphTraversionMarkerReset()
 
         # reset status variables
-        self.markedSignals = []
+        self.marked_signals = []
         self.level = 0
 
     def place_marker_for_current_level(self, signal):
         # mark the node/signal as being visited (meaning computed)
         signal.graphTraversionMarkerMarkVisited(self.level)
-        self.markedSignals.append(signal)
+        self.marked_signals.append(signal)
 
-    def isSignalAlreadyComputable(self, signal : Signal):
+    # NOTE: unused
+    def is_signal_already_computable(self, signal : Signal):
         return signal.graphTraversionMarkerMarkIsVisited()
 
     # Start backward traversion starting from the given startSignal
-    def backwardTraverseSignalsExec__(self, startSignal : Signal, depthCounter : int, system_context = None):
+    def backwards_traverse_signals_exec__(self, start_signal : Signal, depth_counter : int, current_system):
+        # WARNING: This is a recursive function!
+
+        # the list of signals to compute in correct order 
+        execution_order = []
+
+        # list of signals the computation depends on (the tips of the execution tree)
+        dependency_signals = []
+
+        # the list of simulation input signals required for the computation
+        dependency_signals_simulation_inputs = []
+
+        # For each signgal execution_order there might be a blocks that
+        # has an internal memory. It is required to build a list of those blocks
+        # that need a state update after their output(s) are calculated.
+        blocks_to_update_states = []
+
+        #
+        dependency_signals_through_states = []
+
+
+
 
         #
         # check if the datatype of startSignal is defined
         #
 
-        if startSignal.datatype is None:
-            raise BaseException('Unknown datatype for signal ' + startSignal.name + ': no datatype has been specified or could be determined automatically.')
+        if start_signal.datatype is None:
+            raise BaseException('Unknown datatype for signal ' + start_signal.name + ': no datatype has been specified or could be determined automatically.')
 
 
         #
@@ -385,29 +383,29 @@ class BuildExecutionPath:
 
         
         tabs = ''
-        for i in range(0, depthCounter):
+        for i in range(0, depth_counter):
             tabs += '.  '
 
 
-        if not (isinstance(startSignal, SimulationInputSignal) or isinstance(startSignal, BlockOutputSignal)):
+        if not (isinstance(start_signal, SimulationInputSignal) or isinstance(start_signal, BlockOutputSignal)):
             
             # this case must be an error..                  
-            raise BaseException('not implemented or internal error: unexpected type of signal ' + startSignal.name)
+            raise BaseException('not implemented or internal error: unexpected type of signal ' + start_signal.name)
 
         # check if the signal is a system input signal
-        is_crossing_simulation_border = startSignal.is_crossing_system_boundary(self.system) #  self.system != startSignal.sim
+        is_crossing_simulation_border = start_signal.is_crossing_system_boundary(current_system) #  self.system != startSignal.sim
 
-        # TODO: IMPLEMENT: except when startSignal is a simulation input (in this case it is not comuted)
+        # TODO: IMPLEMENT: except when startSignal is a simulation input (in this case it is not computed)
         #  and not isinstance(startSignal, SimulationInputSignal)
-        if startSignal.graphTraversionMarkerMarkIsVisitedOnLevelLowerThan(self.level):
+        if start_signal.graphTraversionMarkerMarkIsVisitedOnLevelLowerThan(self.level):
             # - a previously computed signal has been reached
 
             if self._show_print > 1:
                 print(Style.DIM + tabs + "has already been calculated in a previous traversion") 
 
-            self.dependencySignals.append( startSignal )
+            dependency_signals.append( start_signal )
 
-            # in case startSignal is a simulation input, still add it to the list of simulation input dependiencies
+            # in case startSignal is a simulation input, still add it to the list of simulation input dependencies
             # though it has already been computed
             if is_crossing_simulation_border:
 
@@ -415,64 +413,64 @@ class BuildExecutionPath:
                     print(Style.DIM + tabs + "as it is also a simulation input, adding it to the list of depended inputs")
 
                 # also note down that this is a (actually used) simulation input
-                self.dependencySignalsSimulationInputs.append( startSignal )
+                dependency_signals_simulation_inputs.append( start_signal )
 
-            return
+            return execution_order, dependency_signals, dependency_signals_simulation_inputs, blocks_to_update_states, dependency_signals_through_states
 
-        if startSignal.graphTraversionMarkerMarkIsVisited():
+        if start_signal.graphTraversionMarkerMarkIsVisited():
 
             if self._show_print > 1:
                 print(Style.DIM + tabs + "has already been calculated in this traversion") 
 
-            return
+            return execution_order, dependency_signals, dependency_signals_simulation_inputs, blocks_to_update_states, dependency_signals_through_states
 
         if is_crossing_simulation_border:
-            # signal is an input to the simulation
+            # signal is an input to the system
             # add to the list of dependent inputs
 
             if self._show_print > 1:
                 print(Fore.YELLOW + tabs + "  --> crosses system bounds")
 
-            # startSignal is at the top of the tree, so add it to the dependiencies
-            self.dependencySignals.append( startSignal )
+            # startSignal is at the top of the tree, so add it to the dependencies
+            dependency_signals.append( start_signal )
 
             # also note down that this is a (actually used) simulation input
-            self.dependencySignalsSimulationInputs.append( startSignal )
+            dependency_signals_simulation_inputs.append( start_signal )
 
             if self._show_print > 1:
-                print(Style.DIM + tabs + "added input dependency " + startSignal.toStr())
+                print(Style.DIM + tabs + "added input dependency " + start_signal.toStr())
 
             # mark the node/signal as being visited (meaning computed)
-            self.place_marker_for_current_level(startSignal)
+            self.place_marker_for_current_level(start_signal)
 
-            return
+            return execution_order, dependency_signals, dependency_signals_simulation_inputs, blocks_to_update_states, dependency_signals_through_states
 
 
         # get the blocks prototype function to calculate startSignal
-        block = startSignal.getSourceBlock()
+        block = start_signal.getSourceBlock()
         blocksPrototype = block.getBlockPrototype()
 
         #
         # check if the block that yields startSignal uses internal-states to compute startSignal
         #
 
-        inputsToUpdateStatesTmp = blocksPrototype.config_request_define_state_update_input_dependencies( startSignal )
-        if inputsToUpdateStatesTmp is not None:
+        inputs_to_update_states_tmp = blocksPrototype.config_request_define_state_update_input_dependencies( start_signal )
+        if inputs_to_update_states_tmp is not None:
 
             if self._show_print > 1:
-                print(tabs + "--- signals needed *indirectly* to compute " + startSignal.name + " (through state update) --" )
+                print(tabs + "--- signals needed *indirectly* to compute " + start_signal.name + " (through state update) --" )
 
             # 
-            self.blocksToUpdateStates.append( block )
+            blocks_to_update_states.append( block )
 
             # please note: blocksPrototype.config_request_define_state_update_input_dependencies might return some undetermined signals that are resolved here
-            resolveUndeterminedSignals( inputsToUpdateStatesTmp )
+            resolveUndeterminedSignals( inputs_to_update_states_tmp )
 
             # add the signals that are required to perform the state update
-            self.dependencySignalsThroughStates.extend( inputsToUpdateStatesTmp )
+            dependency_signals_through_states.extend( inputs_to_update_states_tmp )
 
             if self._show_print > 1:
-                for signal in inputsToUpdateStatesTmp:
+                for signal in inputs_to_update_states_tmp:
                     print(Fore.MAGENTA + tabs + "-> S " + signal.name )
 
 
@@ -482,9 +480,9 @@ class BuildExecutionPath:
         #
 
         if self._show_print > 1:
-            print(tabs + "--- signals needed for " + startSignal.name + " --" )
+            print(tabs + "--- signals needed for " + start_signal.name + " --" )
 
-        dependingSignals = blocksPrototype.config_request_define_feedforward_input_dependencies(startSignal)
+        dependingSignals = blocksPrototype.config_request_define_feedforward_input_dependencies(start_signal)
 
         # please note: blocksPrototype.config_request_define_feedforward_input_dependencies might return some undetermined signals that are resolved here
         resolveUndeterminedSignals( dependingSignals )
@@ -497,18 +495,18 @@ class BuildExecutionPath:
             # block startSignal.getSourceBlock() --> startSignal is a starting point
 
             # startSignal is at the top of the tree, so add it to the dependencies
-            self.dependencySignals.append( startSignal )
+            dependency_signals.append( start_signal )
 
             #
             if self._show_print > 1:
-                print(Style.DIM + tabs + "added " + startSignal.toStr())
+                print(Style.DIM + tabs + "added " + start_signal.toStr())
     
-            self.execution_order.append( startSignal )
+            execution_order.append( start_signal )
 
             # mark the node/signal as being visited (meaning computed)
-            self.place_marker_for_current_level(startSignal)
+            self.place_marker_for_current_level(start_signal)
 
-            return
+            return execution_order, dependency_signals, dependency_signals_simulation_inputs, blocks_to_update_states, dependency_signals_through_states
 
 
 
@@ -522,22 +520,36 @@ class BuildExecutionPath:
             if self._show_print > 1:
                 print(Fore.MAGENTA + tabs + "-> S " + signal.name )
 
-            self.backwardTraverseSignalsExec__( signal, depthCounter = depthCounter + 1 )
+            if depth_counter > 100:
+                raise BaseException('maximal number of iterations reached in system ' + signal.system.name + 'signal ' + signal.name)
+
+            # R E C U R S I O N
+            A_execution_order, A_dependency_signals, A_dependency_signals_simulation_inputs, A_blocks_to_update_states, A_dependency_signals_through_states = self.backwards_traverse_signals_exec__( signal, depth_counter = depth_counter + 1, current_system=current_system )
+
+            execution_order.extend(                      A_execution_order )
+            dependency_signals.extend(                   A_dependency_signals )
+            dependency_signals_simulation_inputs.extend( A_dependency_signals_simulation_inputs )
+            blocks_to_update_states.extend(              A_blocks_to_update_states )
+            dependency_signals_through_states.extend(    A_dependency_signals_through_states )
+
 
         #
         # FINALIZE: now also startSignal can be computed
         #
 
         #
-        # store startSignal as reachable (put it on the exeution list)
+        # store startSignal as reachable (put it on the execution list)
         # NOTE: if startSignal is the tip of the tree (no dependingSignals) it is excluded
         #       from this list. However, it is still in the list of dependencySignals.
         #
 
         if self._show_print > 1:
-            print(Style.DIM + tabs + "added " + startSignal.toStr())
+            print(Style.DIM + tabs + "added " + start_signal.toStr())
 
-        self.execution_order.append( startSignal )
+        execution_order.append( start_signal )
 
         # mark the node/signal as being visited (meaning computed)
-        self.place_marker_for_current_level(startSignal)
+        self.place_marker_for_current_level(start_signal)
+
+
+        return execution_order, dependency_signals, dependency_signals_simulation_inputs, blocks_to_update_states, dependency_signals_through_states
