@@ -5,9 +5,9 @@
 #
 
 from .diagram_core.datatypes import DataTypeInt32, DataTypeFloat64, DataTypeBoolean, DataTypeArray, DataTypeNumeric, DataTypePointer
-from .code_generation_templates import TargetGenericCpp, TargetBasicExecutable, TargetWasm
-from .system_context import init_simulation_context, get_system_context, enter_system, leave_system, clear, set_primary_outputs, append_primay_ouput
+from .code_generation_templates import TargetGenericCpp, TargetRawCpp, TargetBasicExecutable, TargetWasm
+from .system_context import init_simulation_context, get_system_context, enter_system, leave_system, clear, set_primary_outputs, append_output, include_cpp_code
 from .standard_library import *
 from .subsystems import sub_if, sub_loop, sub_switch, state_sub, sub_statemachine
 from .high_level_user_commands import *
-from .signal_interface import SignalUserTemplate
+from .signal_interface import SignalUserTemplate, structure
