@@ -82,7 +82,16 @@ def boolean(value):
         return value
     else:
         # create a new constant
-        return dy.const(value, dy.DataTypeBoolean(1) )
+        if type(value) is bool:
+            if value:
+                v = 1
+            else:
+                v = 0
+
+        elif type(value) is int:
+            v = value
+
+        return dy.const(v, dy.DataTypeBoolean(1) )
 
 
 
@@ -335,7 +344,7 @@ def counter():
     if not 'counter' in dy.get_system_context().components:
         # no counter has been defined in this system so far. Hence, create one.
 
-        increase = dy.const(1, dy.DataTypeInt32(1) )
+        increase = dy.const(1, dy.DataTypeInt32(1) ).set_name('cnt_increase')
         cnt = dy.signal()
         tmp = dy.delay(cnt + increase)
         cnt << tmp 
